@@ -1,5 +1,6 @@
 # testing/assertions.py
-# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2015 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -8,79 +9,90 @@ from __future__ import absolute_import
 
 from . import util as testutil
 from sqlalchemy import pool, orm, util
-from sqlalchemy.engine import default, create_engine, url
-from sqlalchemy import exc as sa_exc
+from sqlalchemy.engine import default, url
 from sqlalchemy.util import decorator
-from sqlalchemy import types as sqltypes, schema
+from sqlalchemy import types as sqltypes, schema, exc as sa_exc
 import warnings
 import re
-from .warnings import resetwarnings
 from .exclusions import db_spec, _is_excluded
 from . import assertsql
 from . import config
-import itertools
 from .util import fail
 import contextlib
+from . import mock
+
+
+def expect_warnings(*messages, **kw):
+    """Context manager which expects one or more warnings.
+
+    With no arguments, squelches all SAWarnings emitted via
+    sqlalchemy.util.warn and sqlalchemy.util.warn_limited.   Otherwise
+    pass string expressions that will match selected warnings via regex;
+    all non-matching warnings are sent through.
+
+    The expect version **asserts** that the warnings were in fact seen.
+
+    Note that the test suite sets SAWarning warnings to raise exceptions.
+
+    """
+    return _expect_warnings(sa_exc.SAWarning, messages, **kw)
+
+
+@contextlib.contextmanager
+def expect_warnings_on(db, *messages, **kw):
+    """Context manager which expects one or more warnings on specific
+    dialects.
+
+    The expect version **asserts** that the warnings were in fact seen.
+
+    """
+    spec = db_spec(db)
+
+    if isinstance(db, util.string_types) and not spec(config._current):
+        yield
+    elif not _is_excluded(*db):
+        yield
+    else:
+        with expect_warnings(*messages, **kw):
+            yield
 
 
 def emits_warning(*messages):
-    """Mark a test as emitting a warning.
+    """Decorator form of expect_warnings().
+
+    Note that emits_warning does **not** assert that the warnings
+    were in fact seen.
 
-    With no arguments, squelches all SAWarning failures.  Or pass one or more
-    strings; these will be matched to the root of the warning description by
-    warnings.filterwarnings().
     """
-    # TODO: it would be nice to assert that a named warning was
-    # emitted. should work with some monkeypatching of warnings,
-    # and may work on non-CPython if they keep to the spirit of
-    # warnings.showwarning's docstring.
-    # - update: jython looks ok, it uses cpython's module
 
     @decorator
     def decorate(fn, *args, **kw):
-        # todo: should probably be strict about this, too
-        filters = [dict(action='ignore',
-                        category=sa_exc.SAPendingDeprecationWarning)]
-        if not messages:
-            filters.append(dict(action='ignore',
-                                 category=sa_exc.SAWarning))
-        else:
-            filters.extend(dict(action='ignore',
-                                 message=message,
-                                 category=sa_exc.SAWarning)
-                            for message in messages)
-        for f in filters:
-            warnings.filterwarnings(**f)
-        try:
+        with expect_warnings(assert_=False, *messages):
             return fn(*args, **kw)
-        finally:
-            resetwarnings()
+
     return decorate
 
 
-def emits_warning_on(db, *warnings):
+def expect_deprecated(*messages, **kw):
+    return _expect_warnings(sa_exc.SADeprecationWarning, messages, **kw)
+
+
+def emits_warning_on(db, *messages):
     """Mark a test as emitting a warning on a specific dialect.
 
     With no arguments, squelches all SAWarning failures.  Or pass one or more
     strings; these will be matched to the root of the warning description by
     warnings.filterwarnings().
-    """
-    spec = db_spec(db)
 
+    Note that emits_warning_on does **not** assert that the warnings
+    were in fact seen.
+
+    """
     @decorator
     def decorate(fn, *args, **kw):
-        if isinstance(db, util.string_types):
-            if not spec(config._current):
-                return fn(*args, **kw)
-            else:
-                wrapped = emits_warning(*warnings)(fn)
-                return wrapped(*args, **kw)
-        else:
-            if not _is_excluded(*db):
-                return fn(*args, **kw)
-            else:
-                wrapped = emits_warning(*warnings)(fn)
-                return wrapped(*args, **kw)
+        with expect_warnings_on(db, *messages):
+            return fn(*args, **kw)
+
     return decorate
 
 
@@ -94,38 +106,52 @@ def uses_deprecated(*messages):
     As a special case, you may pass a function name prefixed with //
     and it will be re-written as needed to match the standard warning
     verbiage emitted by the sqlalchemy.util.deprecated decorator.
+
+    Note that uses_deprecated does **not** assert that the warnings
+    were in fact seen.
+
     """
 
     @decorator
     def decorate(fn, *args, **kw):
-        with expect_deprecated(*messages):
+        with expect_deprecated(*messages, assert_=False):
             return fn(*args, **kw)
     return decorate
 
-@contextlib.contextmanager
-def expect_deprecated(*messages):
-    # todo: should probably be strict about this, too
-    filters = [dict(action='ignore',
-                    category=sa_exc.SAPendingDeprecationWarning)]
-    if not messages:
-        filters.append(dict(action='ignore',
-                            category=sa_exc.SADeprecationWarning))
-    else:
-        filters.extend(
-            [dict(action='ignore',
-                  message=message,
-                  category=sa_exc.SADeprecationWarning)
-             for message in
-             [(m.startswith('//') and
-                ('Call to deprecated function ' + m[2:]) or m)
-               for m in messages]])
 
-    for f in filters:
-        warnings.filterwarnings(**f)
-    try:
+@contextlib.contextmanager
+def _expect_warnings(exc_cls, messages, regex=True, assert_=True):
+
+    if regex:
+        filters = [re.compile(msg, re.I) for msg in messages]
+    else:
+        filters = messages
+
+    seen = set(filters)
+
+    real_warn = warnings.warn
+
+    def our_warn(msg, exception, *arg, **kw):
+        if not issubclass(exception, exc_cls):
+            return real_warn(msg, exception, *arg, **kw)
+
+        if not filters:
+            return
+
+        for filter_ in filters:
+            if (regex and filter_.match(msg)) or \
+                    (not regex and filter_ == msg):
+                seen.discard(filter_)
+                break
+        else:
+            real_warn(msg, exception, *arg, **kw)
+
+    with mock.patch("warnings.warn", our_warn):
         yield
-    finally:
-        resetwarnings()
+
+    if assert_:
+        assert not seen, "Warnings were not seen: %s" % \
+            ", ".join("%r" % (s.pattern if regex else s) for s in seen)
 
 
 def global_cleanup_assertions():
@@ -139,6 +165,8 @@ def global_cleanup_assertions():
     _assert_no_stray_pool_connections()
 
 _STRAY_CONNECTION_FAILURES = 0
+
+
 def _assert_no_stray_pool_connections():
     global _STRAY_CONNECTION_FAILURES
 
@@ -150,12 +178,11 @@ def _assert_no_stray_pool_connections():
     # there's a ref in there.  usually just one.
     if pool._refs:
 
-        # OK, let's be somewhat forgiving.  Increment a counter,
-        # we'll allow a couple of these at most.
+        # OK, let's be somewhat forgiving.
         _STRAY_CONNECTION_FAILURES += 1
 
         print("Encountered a stray connection in test cleanup: %s"
-                        % str(pool._refs))
+              % str(pool._refs))
         # then do a real GC sweep.   We shouldn't even be here
         # so a single sweep should really be doing it, otherwise
         # there's probably a real unreachable cycle somewhere.
@@ -164,7 +191,7 @@ def _assert_no_stray_pool_connections():
     # if we've already had two of these occurrences, or
     # after a hard gc sweep we still have pool._refs?!
     # now we have to raise.
-    if _STRAY_CONNECTION_FAILURES >= 2 or pool._refs:
+    if pool._refs:
         err = str(pool._refs)
 
         # but clean out the pool refs collection directly,
@@ -172,7 +199,11 @@ def _assert_no_stray_pool_connections():
         # so the error doesn't at least keep happening.
         pool._refs.clear()
         _STRAY_CONNECTION_FAILURES = 0
-        assert False, "Stray connections in cleanup: %s" % err
+        assert False, "Stray connection refused to leave "\
+            "after gc.collect(): %s" % err
+    elif _STRAY_CONNECTION_FAILURES > 10:
+        assert False, "Encountered more than 10 stray connections"
+        _STRAY_CONNECTION_FAILURES = 0
 
 
 def eq_(a, b, msg=None):
@@ -217,17 +248,19 @@ def assert_raises_message(except_cls, msg, callable_, *args, **kwargs):
         callable_(*args, **kwargs)
         assert False, "Callable did not raise an exception"
     except except_cls as e:
-        assert re.search(msg, util.text_type(e), re.UNICODE), "%r !~ %s" % (msg, e)
+        assert re.search(
+            msg, util.text_type(e), re.UNICODE), "%r !~ %s" % (msg, e)
         print(util.text_type(e).encode('utf-8'))
 
 
 class AssertsCompiledSQL(object):
     def assert_compile(self, clause, result, params=None,
-                        checkparams=None, dialect=None,
-                        checkpositional=None,
-                        use_default_dialect=False,
-                        allow_dialect_select=False,
-                        literal_binds=False):
+                       checkparams=None, dialect=None,
+                       checkpositional=None,
+                       check_prefetch=None,
+                       use_default_dialect=False,
+                       allow_dialect_select=False,
+                       literal_binds=False):
         if use_default_dialect:
             dialect = default.DefaultDialect()
         elif allow_dialect_select:
@@ -243,7 +276,6 @@ class AssertsCompiledSQL(object):
             elif isinstance(dialect, util.string_types):
                 dialect = url.URL(dialect).get_dialect()()
 
-
         kw = {}
         compile_kwargs = {}
 
@@ -267,10 +299,15 @@ class AssertsCompiledSQL(object):
 
         if util.py3k:
             param_str = param_str.encode('utf-8').decode('ascii', 'ignore')
-            print(("\nSQL String:\n" + util.text_type(c) + param_str).encode('utf-8'))
+            print(
+                ("\nSQL String:\n" +
+                 util.text_type(c) +
+                 param_str).encode('utf-8'))
         else:
-            print("\nSQL String:\n" + util.text_type(c).encode('utf-8') + param_str)
-
+            print(
+                "\nSQL String:\n" +
+                util.text_type(c).encode('utf-8') +
+                param_str)
 
         cc = re.sub(r'[\n\t]', '', util.text_type(c))
 
@@ -281,6 +318,8 @@ class AssertsCompiledSQL(object):
         if checkpositional is not None:
             p = c.construct_params(params)
             eq_(tuple([p[x] for x in c.positiontup]), checkpositional)
+        if check_prefetch is not None:
+            eq_(c.prefetch, check_prefetch)
 
 
 class ComparesTables(object):
@@ -295,7 +334,7 @@ class ComparesTables(object):
 
             if strict_types:
                 msg = "Type '%s' doesn't correspond to type '%s'"
-                assert type(reflected_c.type) is type(c.type), \
+                assert isinstance(reflected_c.type, type(c.type)), \
                     msg % (reflected_c.type, c.type)
             else:
                 self.assert_types_base(reflected_c, c)
@@ -317,8 +356,8 @@ class ComparesTables(object):
 
     def assert_types_base(self, c1, c2):
         assert c1.type._compare_type_affinity(c2.type),\
-                "On column %r, type '%s' doesn't correspond to type '%s'" % \
-                (c1.name, c1.type, c2.type)
+            "On column %r, type '%s' doesn't correspond to type '%s'" % \
+            (c1.name, c1.type, c2.type)
 
 
 class AssertsExecutionResults(object):
@@ -362,7 +401,8 @@ class AssertsExecutionResults(object):
         found = util.IdentitySet(result)
         expected = set([immutabledict(e) for e in expected])
 
-        for wrong in util.itertools_filterfalse(lambda o: type(o) == cls, found):
+        for wrong in util.itertools_filterfalse(lambda o:
+                                                isinstance(o, cls), found):
             fail('Unexpected type "%s", expected "%s"' % (
                 type(wrong).__name__, cls.__name__))
 
@@ -393,31 +433,30 @@ class AssertsExecutionResults(object):
             else:
                 fail(
                     "Expected %s instance with attributes %s not found." % (
-                    cls.__name__, repr(expected_item)))
+                        cls.__name__, repr(expected_item)))
         return True
 
-    def assert_sql_execution(self, db, callable_, *rules):
-        assertsql.asserter.add_rules(rules)
-        try:
-            callable_()
-            assertsql.asserter.statement_complete()
-        finally:
-            assertsql.asserter.clear_rules()
+    def sql_execution_asserter(self, db=None):
+        if db is None:
+            from . import db as db
 
-    def assert_sql(self, db, callable_, list_, with_sequences=None):
-        if with_sequences is not None and config.db.dialect.supports_sequences:
-            rules = with_sequences
-        else:
-            rules = list_
+        return assertsql.assert_engine(db)
+
+    def assert_sql_execution(self, db, callable_, *rules):
+        with self.sql_execution_asserter(db) as asserter:
+            callable_()
+        asserter.assert_(*rules)
+
+    def assert_sql(self, db, callable_, rules):
 
         newrules = []
         for rule in rules:
             if isinstance(rule, dict):
                 newrule = assertsql.AllOf(*[
-                    assertsql.ExactSQL(k, v) for k, v in rule.items()
+                    assertsql.CompiledSQL(k, v) for k, v in rule.items()
                 ])
             else:
-                newrule = assertsql.ExactSQL(*rule)
+                newrule = assertsql.CompiledSQL(*rule)
             newrules.append(newrule)
 
         self.assert_sql_execution(db, callable_, *newrules)
