@@ -416,11 +416,14 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "SELECT TOP 0 t.x, t.y FROM t WHERE t.x = :x_1 ORDER BY t.y",
             checkparams={'x_1': 5}
         )
+        c = s.compile(dialect=mssql.MSDialect())
+        eq_(len(c._result_columns), 2)
+        assert t.c.x in set(c._create_result_map()['x'][1])
 
     def test_offset_using_window(self):
         t = table('t', column('x', Integer), column('y', Integer))
 
-        s = select([t]).where(t.c.x==5).order_by(t.c.y).offset(20)
+        s = select([t]).where(t.c.x == 5).order_by(t.c.y).offset(20)
 
         # test that the select is not altered with subsequent compile
         # calls
@@ -430,10 +433,14 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                 "SELECT anon_1.x, anon_1.y FROM (SELECT t.x AS x, t.y "
                 "AS y, ROW_NUMBER() OVER (ORDER BY t.y) AS "
                 "mssql_rn FROM t WHERE t.x = :x_1) AS "
-                "anon_1 WHERE mssql_rn > :mssql_rn_1",
-                checkparams={'mssql_rn_1': 20, 'x_1': 5}
+                "anon_1 WHERE mssql_rn > :param_1",
+                checkparams={'param_1': 20, 'x_1': 5}
             )
 
+            c = s.compile(dialect=mssql.MSDialect())
+            eq_(len(c._result_columns), 2)
+            assert t.c.x in set(c._create_result_map()['x'][1])
+
     def test_limit_offset_using_window(self):
         t = table('t', column('x', Integer), column('y', Integer))
 
@@ -446,9 +453,13 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "ROW_NUMBER() OVER (ORDER BY t.y) AS mssql_rn "
             "FROM t "
             "WHERE t.x = :x_1) AS anon_1 "
-            "WHERE mssql_rn > :mssql_rn_1 AND mssql_rn <= :mssql_rn_2",
-            checkparams={'mssql_rn_1': 20, 'mssql_rn_2': 30, 'x_1': 5}
+            "WHERE mssql_rn > :param_1 AND mssql_rn <= :param_2 + :param_1",
+            checkparams={'param_1': 20, 'param_2': 10, 'x_1': 5}
         )
+        c = s.compile(dialect=mssql.MSDialect())
+        eq_(len(c._result_columns), 2)
+        assert t.c.x in set(c._create_result_map()['x'][1])
+        assert t.c.y in set(c._create_result_map()['y'][1])
 
     def test_limit_offset_with_correlated_order_by(self):
         t1 = table('t1', column('x', Integer), column('y', Integer))
@@ -467,10 +478,15 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             ") AS mssql_rn "
             "FROM t1 "
             "WHERE t1.x = :x_1) AS anon_1 "
-            "WHERE mssql_rn > :mssql_rn_1 AND mssql_rn <= :mssql_rn_2",
-            checkparams={'mssql_rn_1': 20, 'mssql_rn_2': 30, 'x_1': 5}
+            "WHERE mssql_rn > :param_1 AND mssql_rn <= :param_2 + :param_1",
+            checkparams={'param_1': 20, 'param_2': 10, 'x_1': 5}
         )
 
+        c = s.compile(dialect=mssql.MSDialect())
+        eq_(len(c._result_columns), 2)
+        assert t1.c.x in set(c._create_result_map()['x'][1])
+        assert t1.c.y in set(c._create_result_map()['y'][1])
+
     def test_limit_zero_offset_using_window(self):
         t = table('t', column('x', Integer), column('y', Integer))
 
@@ -510,6 +526,29 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                             "CREATE TABLE test (id INTEGER NOT NULL IDENTITY(1,1))"
                             )
 
+    def test_table_pkc_clustering(self):
+        metadata = MetaData()
+        tbl = Table('test', metadata,
+                    Column('x', Integer, autoincrement=False),
+                    Column('y', Integer, autoincrement=False),
+                    PrimaryKeyConstraint("x", "y", mssql_clustered=True))
+        self.assert_compile(schema.CreateTable(tbl),
+                            "CREATE TABLE test (x INTEGER NOT NULL, y INTEGER NOT NULL, "
+                            "PRIMARY KEY CLUSTERED (x, y))"
+                            )
+
+    def test_table_uc_clustering(self):
+        metadata = MetaData()
+        tbl = Table('test', metadata,
+                    Column('x', Integer, autoincrement=False),
+                    Column('y', Integer, autoincrement=False),
+                    PrimaryKeyConstraint("x"),
+                    UniqueConstraint("y", mssql_clustered=True))
+        self.assert_compile(schema.CreateTable(tbl),
+                            "CREATE TABLE test (x INTEGER NOT NULL, y INTEGER NULL, "
+                            "PRIMARY KEY (x), UNIQUE CLUSTERED (y))"
+                            )
+
     def test_index_clustering(self):
         metadata = MetaData()
         tbl = Table('test', metadata,
@@ -538,6 +577,17 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "CREATE INDEX bar ON foo (x > 5)"
         )
 
+    def test_drop_index_w_schema(self):
+        m = MetaData()
+        t1 = Table('foo', m,
+                Column('x', Integer),
+                schema='bar'
+            )
+        self.assert_compile(
+            schema.DropIndex(Index("idx_foo", t1.c.x)),
+            "DROP INDEX idx_foo ON bar.foo"
+        )
+
     def test_index_extra_include_1(self):
         metadata = MetaData()
         tbl = Table('test', metadata,
